/*
 * Kernel Debugger Architecture Independent Support Functions
 *
 * Copyright (C) 2002 Intransa, Inc.
 * Copyright (C) Paul Lai (paul.lai@intransa.com)
 *   leveraging heavily from arch/i386/kdb/kdbasupport.c
 *
 */

#include <linux/config.h>
#include <linux/string.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/ptrace.h>
#include <linux/mm.h>
#include <linux/sched.h>
#include <linux/kdb.h>
#include <linux/kdbprivate.h>

#include <asm/processor.h>
#if 0
#include <asm/msr.h>
#endif
#include <asm/uaccess.h>
#include <asm/processor.h>
#include <asm/branch.h>

#define DEBUG_KDBASUPPORT	0

/*
 * kdba_find_return
 *
 *	Given a starting point on the stack and symtab data for the
 *	current function, scan up the stack looking for a return
 *	address for this function.
 * Inputs:
 *	sp	Starting stack pointer for scan
 *	ss	Start of stack for current process
 *	symtab	kallsyms symbol data for the function
 * Outputs:
 *	None.
 * Returns:
 *	Position on stack of return address, 0 if not found.
 * Locking:
 *	None.
 * Remarks:
 *	This is sensitive to the calling sequence generated by gcc.
 */

static kdb_machreg_t
kdba_find_return(kdb_machreg_t sp, kdb_machreg_t ss, const kdb_symtab_t *symtab)
{
	return(0);
}

/*
 * kdba_prologue
 *
 */

int
kdba_prologue(const kdb_symtab_t *symtab, kdb_machreg_t pc, kdb_machreg_t sp,
	      kdb_machreg_t fp, kdb_machreg_t ss, int caller, kdb_ar_t *ar)
{
	return(1);
}


void
kdba_installdbreg(kdb_bp_t *bp)
{
	return;
}

void
kdba_removedbreg(kdb_bp_t *bp)
{
}

kdb_machreg_t
kdba_getdr(int regnum)
{
	kdb_machreg_t contents = 0;
	return contents;
}


kdb_machreg_t
kdb_getcr(int regnum)
{
	kdb_machreg_t contents = 0;
	return contents;
}

void
kdba_putdr(int regnum, kdb_machreg_t contents)
{
}

/*
 * kdba_getregcontents
 *
 *	Return the contents of the register specified by the
 *	input string argument.   Return an error if the string
 *	does not match a machine register.
 *
 *	The following pseudo register names are supported:
 *	   &regs	 - Prints address of exception frame
 *	   kesp		 - Prints kernel stack pointer at time of fault
 *	   cesp		 - Prints current kernel stack pointer, inside kdb
 *	   ceflags	 - Prints current flags, inside kdb
 *	   %<regname>	 - Uses the value of the registers at the
 *			   last time the user process entered kernel
 *			   mode, instead of the registers at the time
 *			   kdb was entered.
 *
 * Parameters:
 *	regname		Pointer to string naming register
 *	regs		Pointer to structure containing registers.
 * Outputs:
 *	*contents	Pointer to unsigned long to recieve register contents
 * Returns:
 *	0		Success
 *	KDB_BADREG	Invalid register name
 * Locking:
 * 	None.
 * Remarks:
 * 	If kdb was entered via an interrupt from the kernel itself then
 *	ss and esp are *not* on the stack.
 */

static struct kdbregs {
	char   *reg_name;
	size_t	reg_offset;
} kdbreglist[] = {
	{ "zero",	offsetof(struct pt_regs, regs[0]) },
	{ "at",		offsetof(struct pt_regs, regs[1]) },
	{ "v0",		offsetof(struct pt_regs, regs[2]) },
	{ "v1",		offsetof(struct pt_regs, regs[3]) },
	{ "a0",		offsetof(struct pt_regs, regs[4]) },
	{ "a1",		offsetof(struct pt_regs, regs[5]) },
	{ "a2",		offsetof(struct pt_regs, regs[6]) },
	{ "a3",		offsetof(struct pt_regs, regs[7]) },
	{ "t0",		offsetof(struct pt_regs, regs[8]) },
	{ "t1",		offsetof(struct pt_regs, regs[9]) },
	{ "t2",		offsetof(struct pt_regs, regs[10]) },
	{ "t3",		offsetof(struct pt_regs, regs[11]) },
	{ "t4",		offsetof(struct pt_regs, regs[12]) },
	{ "t5",		offsetof(struct pt_regs, regs[13]) },
	{ "t6",		offsetof(struct pt_regs, regs[14]) },
	{ "t7",		offsetof(struct pt_regs, regs[15]) },
	{ "s0",		offsetof(struct pt_regs, regs[16]) },
	{ "s1",		offsetof(struct pt_regs, regs[17]) },
	{ "s2",		offsetof(struct pt_regs, regs[18]) },
	{ "s3",		offsetof(struct pt_regs, regs[19]) },
	{ "s4",		offsetof(struct pt_regs, regs[20]) },
	{ "s5",		offsetof(struct pt_regs, regs[21]) },
	{ "s6",		offsetof(struct pt_regs, regs[22]) },
	{ "s7",		offsetof(struct pt_regs, regs[23]) },
	{ "t8",		offsetof(struct pt_regs, regs[24]) },
	{ "t9",		offsetof(struct pt_regs, regs[25]) },
	{ "k0",		offsetof(struct pt_regs, regs[26]) },
	{ "k1",		offsetof(struct pt_regs, regs[27]) },
	{ "gp",		offsetof(struct pt_regs, regs[28]) },
	{ "sp",		offsetof(struct pt_regs, regs[29]) },
	{ "s8",		offsetof(struct pt_regs, regs[30]) },
	{ "ra",		offsetof(struct pt_regs, regs[31]) },
	{ "hi",		offsetof(struct pt_regs, regs[31]) },
	{ "lo",		offsetof(struct pt_regs, regs[31]) },
	{ "cp0_epc",	offsetof(struct pt_regs, cp0_epc) },
	{ "cp0_badvaddr",	offsetof(struct pt_regs, cp0_badvaddr) },
	{ "cp0_status",		offsetof(struct pt_regs, cp0_status) },
	{ "cp0_cause",		offsetof(struct pt_regs, cp0_cause) },
};

static const int nkdbreglist = sizeof(kdbreglist) / sizeof(struct kdbregs);

static struct kdbregs dbreglist[] = {
	{ "dr0", 	0 },
	{ "dr1", 	1 },
	{ "dr2",	2 },
	{ "dr3", 	3 },
	{ "dr6", 	6 },
	{ "dr7",	7 },
};

static const int ndbreglist = sizeof(dbreglist) / sizeof(struct kdbregs);

static int
__atoi( char *c)
{
	int	ret=0;

	for ( ;*c != 0; c++)
	{
		if ( *c >= '0' && *c <= '9')
		{
			ret *= 10 ;
			ret += ( *c - '0' );
		}
	}
	return ret;
}

int
kdba_getregcontents(const char *regname,
		    struct pt_regs *regs,
		    kdb_machreg_t *contents)
{
	int i;
	int regno = 0;

	if (strcmp(regname, "&regs") == 0) {
		*contents = (unsigned long)regs;
		return 0;
	}

	for (i=0; i<nkdbreglist; i++) {
		if (strnicmp(kdbreglist[i].reg_name,
			     regname,
			     strlen(regname)) == 0)
			break;
	}

	if ((i < nkdbreglist)
	 && (strlen(kdbreglist[i].reg_name) == strlen(regname))) {
		*contents = *(unsigned long *)((unsigned long)regs +
				kdbreglist[i].reg_offset);
		return(0);
	}

	return KDB_BADREG;
}

/*
 * kdba_setregcontents
 *
 *	Set the contents of the register specified by the
 *	input string argument.   Return an error if the string
 *	does not match a machine register.
 *
 *	Supports modification of user-mode registers via
 *	%<register-name>
 *
 * Parameters:
 *	regname		Pointer to string naming register
 *	regs		Pointer to structure containing registers.
 *	contents	Unsigned long containing new register contents
 * Outputs:
 * Returns:
 *	0		Success
 *	KDB_BADREG	Invalid register name
 * Locking:
 * 	None.
 * Remarks:
 */

int
kdba_setregcontents(const char *regname,
		  struct pt_regs *regs,
		  unsigned long contents)
{
	int i;


	for (i=0; i<nkdbreglist; i++) {
		if (strnicmp(kdbreglist[i].reg_name,
			     regname,
			     strlen(regname)) == 0)
			break;
	}

	if ((i < nkdbreglist)
	 && (strlen(kdbreglist[i].reg_name) == strlen(regname))) {
		*(unsigned long *)((unsigned long)regs
				   + kdbreglist[i].reg_offset) = contents;
		return 0;
	}

	return KDB_BADREG;
}

/*
 * kdba_dumpregs
 *
 *	Dump the specified register set to the display.
 *
 * Parameters:
 *	regs		Pointer to structure containing registers.
 *	type		Character string identifying register set to dump
 *	extra		string further identifying register (optional)
 * Outputs:
 * Returns:
 *	0		Success
 * Locking:
 * 	None.
 * Remarks:
 *	This function will dump the general register set if the type
 *	argument is NULL (struct pt_regs).   The alternate register
 *	set types supported by this function:
 *
 *	d 		Debug registers
 *	c		Control registers
 *	u		User registers at most recent entry to kernel
 * Following not yet implemented:
 *	m		Model Specific Registers (extra defines register #)
 *	r		Memory Type Range Registers (extra defines register)
 */

int
kdba_dumpregs(struct pt_regs *regs,
	    const char *type,
	    const char *extra)
{
	int i=0;
	int count = 0;
	kdb_machreg_t contents;
	char	*ltype = type;
	int regno = 0;

	if (ltype == NULL) {
		struct kdbregs *rlp;

		for (i=0, rlp=kdbreglist; i<nkdbreglist; i++,rlp++) {
			kdb_printf("%s = ", rlp->reg_name);
			kdba_getregcontents(rlp->reg_name, regs, &contents);
			kdb_printf("0x%08lx ", contents);
			if ((++count % 4) == 0)
				kdb_printf("\n");
		}
#if 0
		kdb_printf("lo = 0x%08lx ", regs->lo);
		kdb_printf("hi = 0x%08lx ", regs->hi);
		kdb_printf("cp0_epc = 0x%08lx ", regs->cp0_epc);
		kdb_printf("\n");
		kdb_printf("cp0_badvaddr = 0x%08lx ", regs->cp0_badvaddr);
		kdb_printf("cp0_status = 0x%08lx ", regs->cp0_status);
		kdb_printf("cp0_cause = 0x%08lx ", regs->cp0_cause);
		kdb_printf("\n");
#endif

		kdb_printf("&regs = 0x%p\n", regs);

		return 0;
	}

	if (ltype[0] == '&' && strcmp(ltype,"&regs")==0)
	{
		kdba_getregcontents(ltype, regs, &contents);
		if (contents = (int) regs)
			kdb_printf("&regs = 0x%p\n", contents);

		return 0;
	}


	if (ltype[0] == '$')
	{
		regno = __atoi(ltype+1);
		if (regno < 32 && regno >= 0)
			ltype = kdbreglist[regno].reg_name ;
	}


	switch (ltype[0]) {
	case 'a':
	case 'c':
	case 'g':
	case 'r':
	case 's':
	case 't':
	case 'v':
	case 'z':
		{
		struct kdbregs *rlp;
		for (i=0, rlp=kdbreglist; i<nkdbreglist; i++,rlp++)
			if (strcmp(rlp->reg_name, ltype) == 0)
			{
				kdb_printf("%s = ", ltype);
				kdba_getregcontents(ltype, regs, &contents);
				kdb_printf("0x%08lx ", contents);
				kdb_printf("\n");
				return 0;
			}
		}
	default:
		return KDB_BADREG;
	}

	/* NOTREACHED */
	return 0;
}

kdb_machreg_t
kdba_getpc(kdb_eframe_t ef)
{
#if DEBUG_KDBASUPPORT
kdb_printf("kdba_getpc(): ef=0x%x \n",ef);
#endif
	return ef->cp0_epc;
}

int
kdba_setpc(kdb_eframe_t ef, kdb_machreg_t newpc)
{
#if DEBUG_KDBASUPPORT
kdb_printf("kdba_setpc(): ef=0x%x newpc=0x%x\n",ef, newpc);
#endif
	ef->cp0_epc = newpc;
	KDB_STATE_SET(IP_ADJUSTED);

	return 0;
}

/*
 * kdba_main_loop
 *
 *	Do any architecture specific set up before entering the main kdb loop.
 *	The primary function of this routine is to make all processes look the
 *	same to kdb, kdb must be able to list a process without worrying if the
 *	process is running or blocked, so make all process look as though they
 *	are blocked.
 *
 * Inputs:
 *	reason		The reason KDB was invoked
 *	error		The hardware-defined error code
 *	error2		kdb's current reason code.  Initially error but can change
 *			acording to kdb state.
 *	db_result	Result from break or debug point.
 *	ef		The exception frame at time of fault/breakpoint.  If reason
 *			is KDB_REASON_SILENT then ef is NULL, otherwise it should
 *			always be valid.
 * Returns:
 *	0	KDB was invoked for an event which it wasn't responsible
 *	1	KDB handled the event for which it was invoked.
 * Outputs:
 *	Sets eip and esp in current->thread.
 * Locking:
 *	None.
 * Remarks:
 *	none.
 */

int
kdba_main_loop(kdb_reason_t reason, kdb_reason_t reason2, int error,
	       kdb_dbtrap_t db_result, kdb_eframe_t ef)
{
#if DEBUG_KDBASUPPORT
kdb_printf("kdba_main_loop():\n");
#endif
#if 0
	if (ef) {
		kdba_getregcontents("sp", ef, &(current->thread.reg29));
	}
#endif
	return(kdb_main_loop(reason, reason2, error, db_result, ef));
}

void
kdba_disableint(kdb_intstate_t *state)
{
	int *fp = (int *)state;
	int   flags;

// kdb_printf("kdba_disableint():\n");
	__save_flags(flags);
	__cli();

	*fp = flags;
}

void
kdba_restoreint(kdb_intstate_t *state)
{
	int flags = *(int *)state;
	__restore_flags(flags);
// kdb_printf("kdba_restoreint():\n");
}

void
kdba_setsinglestep(struct pt_regs *regs)
{
#if DEBUG_KDBASUPPORT
kdb_printf("kdba_setsinglestep(): before cp0_status=0x%x cp0_cause=0x%x\n",
		regs->cp0_status,
		regs->cp0_cause );
#endif

//	regs->cp0_cause |= 0x100  ;
//	regs->cp0_status |= ST0_ERL ;
//	regs->cp0_epc += 4 ;

#if DEBUG_KDBASUPPORT
kdb_printf("kdba_setsinglestep(): after cp0_status=0x%x cp0_cause=0x%x\n",
		regs->cp0_status,
		regs->cp0_cause );
#endif

#if 0
	if (regs->eflags & EF_IE)
		KDB_STATE_SET(A_IF);
	else
		KDB_STATE_CLEAR(A_IF);
	regs->eflags = (regs->eflags | EF_TF) & ~EF_IE;
#endif
}

void
kdba_clearsinglestep(struct pt_regs *regs)
{
#if DEBUG_KDBASUPPORT
kdb_printf("kdba_clearsinglestep():\n");
#endif

#if 0
	if (KDB_STATE(A_IF))
		regs->eflags |= EF_IE;
	else
		regs->eflags &= ~EF_IE;
#endif
}

int
kdba_getcurrentframe(struct pt_regs *regs)
{
#if DEBUG_KDBASUPPORT
kdb_printf("kdba_getcurrentframe():\n");
#endif
#if 0
	regs->xcs = 0;
#if defined(CONFIG_FRAME_POINTER)
	asm volatile("movl %%ebp,%0":"=m" (*(int *)&regs->ebp));
#endif
	asm volatile("movl %%esp,%0":"=m" (*(int *)&regs->esp));

#endif
	return 0;
}

#ifdef KDB_HAVE_LONGJMP
int
kdba_setjmp(kdb_jmp_buf *jb)
{
// kdb_printf("kdba_setjmp(0x%x):\n", jb);
	__asm__(
		".set noat\n"
		"\t.set noreorder\n"
		"\tsw $0, 0(%0)\n"
		"\tsw $1, 4(%0)\n"
		"\tsw $2, 8(%0)\n"
		"\tsw $3, 12(%0)\n"
		"\tsw $4, 16(%0)\n"
		"\tsw $5, 20(%0)\n"
		"\tsw $6, 24(%0)\n"
		"\tsw $7, 28(%0)\n"
		"\tsw $8, 32(%0)\n"
		"\tsw $9, 36(%0)\n"
		"\tsw $10, 40(%0)\n"
		"\tsw $11, 44(%0)\n"
		"\tsw $12, 48(%0)\n"
		"\tsw $13, 52(%0)\n"
		"\tsw $14, 56(%0)\n"
		"\tsw $15, 60(%0)\n"
		"\tsw $16, 64(%0)\n"
		"\tsw $17, 68(%0)\n"
		"\tsw $18, 72(%0)\n"
		"\tsw $19, 76(%0)\n"
		"\tsw $20, 80(%0)\n"
		"\tsw $21, 84(%0)\n"
		"\tsw $22, 88(%0)\n"
		"\tsw $23, 92(%0)\n"
		"\tsw $24, 96(%0)\n"
		"\tsw $25, 100(%0)\n"
		"\tsw $26, 104(%0)\n"
		"\tsw $27, 108(%0)\n"
		"\tsw $28, 112(%0)\n"
		"\tsw $29, 116(%0)\n"
		"\tsw $30, 120(%0)\n"
		"\tsw $31, 124(%0)\n"
		"\t.set at\n"
		:: "r" (jb)
	);
	KDB_STATE_SET(LONGJMP);
	return 0;
}

void
kdba_longjmp(kdb_jmp_buf *jb, int reason)
{
// kdb_printf("kdba_longjmp( 0x%x, %d)\n", jb, reason);
#if 0
	__asm__(
		".set noat\n"
		"\t.set noreorder\n"
		"\tmove $16, %0\n"
		"\tlw $0, 0($16)\n"
		"\tlw $1, 4($16)\n"
		"\tlw $3, 12($16)\n"
		"\tlw $4, 16($16)\n"
		"\tlw $5, 20($16)\n"
		"\tlw $6, 24($16)\n"
		"\tlw $7, 28($16)\n"
		"\tlw $8, 32($16)\n"
		"\tlw $9, 36($16)\n"
		"\tlw $10, 40($16)\n"
		"\tlw $11, 44($16)\n"
		"\tlw $12, 48($16)\n"
		"\tlw $13, 52($16)\n"
		"\tlw $14, 56($16)\n"
		"\tlw $15, 60($16)\n"
		"\tlw $17, 68($16)\n"
		"\tlw $18, 72($16)\n"
		"\tlw $19, 76($16)\n"
		"\tlw $20, 80($16)\n"
		"\tlw $21, 84($16)\n"
		"\tlw $22, 88($16)\n"
		"\tlw $23, 92($16)\n"
		"\tlw $24, 96($16)\n"
		"\tlw $25, 100($16)\n"
		"\tlw $26, 104($16)\n"
		"\tlw $27, 108($16)\n"
		"\tlw $28, 112($16)\n"
		"\tlw $30, 120($16)\n"
		"\tlw $31, 124($16)\n"
		"\tlw $2, %1\n"
		"\tlw $29, 116($16)\n"
		"\tlw $16, 64($16)\n"
		"\tjr $31 \n"
		"\tnop\n"
		"\t.set at\n"
		: : "r" (jb) , "m" (reason)
		: "$16"
	);
#endif
}
#endif	/* KDB_HAVE_LONGJMP */


/*
 * kdba_enable_mce
 *
 *	This function is called once on each CPU to enable machine
 *	check exception handling.
 *
 * Inputs:
 *	None.
 * Outputs:
 *	None.
 * Returns:
 *	None.
 * Locking:
 *	None.
 * Remarks:
 *
 */

void
kdba_enable_mce(void)
{
	/* No longer required, arch/i386/kernel/bluesmoke.c does the job now */
}

/*
 * kdba_enable_lbr
 *
 *	Enable last branch recording.
 *
 * Parameters:
 *	None.
 * Returns:
 *	None
 * Locking:
 *	None
 * Remarks:
 *	None.
 */

static unsigned char lbr_warned;

void
kdba_enable_lbr(void)
{
// kdb_printf("kdba_enable_lbr():\n");
#if 0
	u32  lv, hv;

	if (!test_bit(X86_FEATURE_MCA, &boot_cpu_data.x86_capability)) {
		if (lbr_warned) {
			kdb_printf("kdb: machine does not support last branch recording\n");
			lbr_warned = 1;
		}
		return;
	}
	rdmsr(MSR_IA32_DEBUGCTLMSR, lv, hv);
	lv |= 0x1;	/* Set LBR enable */
	wrmsr(MSR_IA32_DEBUGCTLMSR, lv, hv);
#endif
}

/*
 * kdba_disable_lbr
 *
 *	disable last branch recording.
 *
 * Parameters:
 *	None.
 * Returns:
 *	None
 * Locking:
 *	None
 * Remarks:
 *	None.
 */

void
kdba_disable_lbr(void)
{
// kdb_printf("kdba_disable_lbr():\n");
#if 0
	u32  lv, hv;

	if (!test_bit(X86_FEATURE_MCA, &boot_cpu_data.x86_capability)) {
		if (lbr_warned) {
			kdb_printf("kdb: machine does not support last branch recording\n");
			lbr_warned = 1;
		}
		return;
	}
	rdmsr(MSR_IA32_DEBUGCTLMSR, lv, hv);
	lv &= ~0x1;	/* Set LBR disable */
	wrmsr(MSR_IA32_DEBUGCTLMSR, lv, hv);
#endif
}

/*
 * kdba_print_lbr
 *
 *	Print last branch and last exception addresses
 *
 * Parameters:
 *	None.
 * Returns:
 *	None
 * Locking:
 *	None
 * Remarks:
 *	None.
 */

void
kdba_print_lbr(void)
{
// kdb_printf("kdba_print_lbr():\n");
#if 0
	u32  from, to, dummy;

	if (!test_bit(X86_FEATURE_MCA, &boot_cpu_data.x86_capability))
		return;

	rdmsr(MSR_IA32_LASTBRANCHFROMIP, from, dummy);
	rdmsr(MSR_IA32_LASTBRANCHTOIP, to, dummy);
	kdb_printf("Last Branch IP, from: ");
	kdb_symbol_print(from, NULL, KDB_SP_DEFAULT);
	kdb_printf(" to: ");
	kdb_symbol_print(to, NULL, KDB_SP_DEFAULT);
	kdb_printf("\n");
	rdmsr(MSR_IA32_LASTINTFROMIP, from, dummy);
	rdmsr(MSR_IA32_LASTINTTOIP, to, dummy);
	kdb_printf("Last Int IP, from: ");
	kdb_symbol_print(from, NULL, KDB_SP_DEFAULT);
	kdb_printf(" to: ");
	kdb_symbol_print(to, NULL, KDB_SP_DEFAULT);
	kdb_printf("\n");
#endif
}

/*
 * kdba_init
 *
 * 	Architecture specific initialization.
 *
 * Parameters:
 *	None.
 * Returns:
 *	None.
 * Locking:
 *	None.
 * Remarks:
 *	None.
 */

void __init
kdba_init(void)
{
// kdb_printf("kdba_init():\n");
	kdba_enable_lbr();

	return;
}

/*
 * kdba_adjust_ip
 *
 * 	Architecture specific adjustment of instruction pointer before leaving
 *	kdb.
 *
 * Parameters:
 *	reason		The reason KDB was invoked
 *	error		The hardware-defined error code
 *	ef		The exception frame at time of fault/breakpoint.  If reason
 *			is KDB_REASON_SILENT then ef is NULL, otherwise it should
 *			always be valid.
 * Returns:
 *	None.
 * Locking:
 *	None.
 * Remarks:
 *	noop on ix86.
 */

void
kdba_adjust_ip(kdb_reason_t reason, int error, kdb_eframe_t ef)
{
	int	cpu = smp_processor_id();
	if (ef)
	{
#if DEBUG_KDBASUPPORT
kdb_printf("kdba_adjust_ip(): cpu[%d] before adjust ef->cp0_epc=0x%x"
		" ef->cp0_cause=0x%x ef->cp0_status=0x%x\n",
		cpu,
		ef->cp0_epc, ef->cp0_cause, ef->cp0_status );
#endif
		if (reason == KDB_REASON_BREAK)
		{
			ef->cp0_epc -= 4;
			if (delay_slot(ef))
				ef->cp0_epc -= 8;
		}
#if DEBUG_KDBASUPPORT
kdb_printf("kdba_adjust_ip(): cpu[%d] after adjust ef->cp0_epc=0x%x"
		" ef->cp0_cause=0x%x ef->cp0_status=0x%x\n",
		cpu,
		ef->cp0_epc, ef->cp0_cause, ef->cp0_status );
#endif
	}
	return;
}

static inline void
cp_thread_2_pt_regs( struct thread_struct *t, struct pt_regs *regs)
{
	regs->regs[16] = t->reg16;
	regs->regs[17] = t->reg17;
	regs->regs[18] = t->reg18;
	regs->regs[19] = t->reg19;
	regs->regs[20] = t->reg20;
	regs->regs[21] = t->reg21;
	regs->regs[22] = t->reg22;
	regs->regs[23] = t->reg23;
	regs->regs[29] = t->reg29;
	regs->regs[30] = t->reg30;
	regs->regs[31] = t->reg31;

//	no hi/lo in thread_struct
//	regs->lo = thread_saved_pc(t);
//	regs->hi = thread_saved_pc(t);

	regs->cp0_epc = thread_saved_pc(t);

//	regs->cp0_badvaddr = t->cp0_badvaddr; /* user fault */
	regs->cp0_badvaddr = t->cp0_baduaddr; /* kernel fault */
	regs->cp0_status = t->cp0_status;
//
//	no cause in thread_struct
//	regs->cp0_cause = t->cp0_status;

}

int
kdba_rdp(struct task_struct *p,  const char *type, const char *extra)
{

	struct pt_regs taskregs;

	memset(&taskregs,0, sizeof(taskregs));

	cp_thread_2_pt_regs( &p->thread, &taskregs);

	return kdba_dumpregs(&taskregs, type, extra);
}

int
kdba_rda(struct thread_struct *t,  const char *type, const char *extra)
{
	struct pt_regs taskregs;

	memset(&taskregs,0, sizeof(taskregs));

	cp_thread_2_pt_regs( t, &taskregs);

	return kdba_dumpregs(&taskregs, type, extra);
}


void
kdba_bogus()
{
	printk("kdba_bogus\n");
}

void kdba_enter(kdb_reason_t reason, int error, kdb_eframe_t ef)
{
#if 0
	extern int sb1250_kdb_pref_port;
	extern int sb1250_kdb_switch(int port);

	if (reason != KDB_REASON_SILENT)
		sb1250_kdb_switch(sb1250_kdb_pref_port);
	return;
#endif
}
