#ifndef _LINUX_TIVOCONFIG_INIT_H
#define _LINUX_TIVOCONFIG_INIT_H

/*
 * Initialization array for the TiVo configuration registry
 *
 * Copyright (C) 2001-2004 TiVo Inc. All Rights Reserved.
 */

typedef struct {
	TivoConfigSelector selector;
	TivoConfigValue value;
} TivoConfig;


/*
  TEMPERATURE_PARAMS takes shutdown, critical, log, and target temperature
  values and turns them into a 32 bit value.
*/
#define TEMPERATURE_PARAMS(s,c,l,t)                 \
  ((((unsigned)(s))<<24) | (((unsigned)(c))<<16) |      \
   (((unsigned)(l))<<8)  |  ((unsigned)(t)))

static TivoConfig gClassicConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDClassic },	
    { kTivoConfigMemSize, 0x01000000 }, // 16 MB
    { kTivoConfigI2cType, kTivoConfigI2cTypeSingleMSBus },
    { kTivoConfigIdeCap, 0 },
    { kTivoConfigTempAdj, 0x01000000 }, // unity scalar, should probably review
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(75,70,65,48) },
    { kTivoConfigDigitalTunerCnt, 0 },
    { kTivoConfigAnalogTunerCnt, 1 },
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },
    { kTivoConfigDigitalVideoOutputCnt, 0 },
    { 0,0 }
};
static TivoConfig gComboConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDCombo },	
    { kTivoConfigMemSize, 0x02000000 }, // 32 MB
    { kTivoConfigI2cType, kTivoConfigI2cTypeSingleMSBus },
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |
                         kTivoConfigIdeCapSG |
                         kTivoConfigIdeCapScramble },
    { kTivoConfigTempAdj, 0x01000000 }, // unity scalar, by definition
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(75,70,65,48) },
    { kTivoConfigDigitalTunerCnt, 2 },
    { kTivoConfigAnalogTunerCnt, 0 },
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportDTV30 },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportDTV },
    { kTivoConfigDigitalVideoOutputCnt, 0 },
    { 0,0 }
};

// Sorry for the macro abuse, but I got tired of having to change 8
// different tables for every little change...
//
#define BCM_UMA_BASE \
    { kTivoConfigI2cType, kTivoConfigI2cTypeDualMSBus },    \
    { kTivoConfigTempAdj, 0x01000000 }, /* unity scalar */  \
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |            \
                         kTivoConfigIdeCapSG |              \
                         kTivoConfigIdeCapScramble },

#define BCM_UMA_ALT_BASE \
    { kTivoConfigI2cType, kTivoConfigI2cTypeDualMSBusHD },  \
    { kTivoConfigTempAdj, 0x01000000 }, /* unity scalar */  \
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |            \
                         kTivoConfigIdeCapSG |              \
                         kTivoConfigIdeCapScramble },

#define BCM_UMA_SA_MEM \
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */         \
    { kTivoConfigContigmemRegion1, 0x00100000 }, /* 1.0MB */\
    { kTivoConfigContigmemRegion2, 0x00080000 }, /* 512KB */\
    { kTivoConfigContigmemRegion8, 0x00580000 }, /* 5.5MB */

#define BCM_UMA_HD_SA_MEM \
    { kTivoConfigMemSize, 0x04000000 }, /* 64 MB */         \
    { kTivoConfigContigmemRegion1, 0x00800000 }, /* 8.0MB */\
    { kTivoConfigContigmemRegion2, 0x00100000 }, /* 1.0MB */\
    { kTivoConfigContigmemRegion8, 0x01400000 }, /* 20MB  */

#define BCM_UMA_SA_NO_FANCTL \
    { kTivoConfigDigitalTunerCnt, 0 },                      \
    { kTivoConfigAnalogTunerCnt, 1 },                       \
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },   \
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },   \
    { kTivoConfigDigitalVideoOutputCnt, 0 },


#define BCM_UMA_SA \
    BCM_UMA_SA_MEM					    \
    BCM_UMA_SA_NO_FANCTL                                    \
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) },

#define BCM_UMA_HD_SA \
    BCM_UMA_HD_SA_MEM					    \
    BCM_UMA_SA_NO_FANCTL                                    \
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) },

#define BCM_UMA_DTV_MEM \
    { kTivoConfigMemSize, 0x04000000 }, /* 64 MB */         \
    { kTivoConfigContigmemRegion1, 0x00800000 }, /* 8.0MB */\
    { kTivoConfigContigmemRegion2, 0x00100000 }, /* 1.0MB */\
    { kTivoConfigContigmemRegion8, 0x00a00000 },  /*10.0MB */

#define BCM_UMA_HD_DTV_PX_MEM \
    { kTivoConfigMemSize, 0x04000000 }, /* 64 MB */         \
    { kTivoConfigContigmemRegion1, 0x00C00000 }, /*12.0MB */\
    { kTivoConfigContigmemRegion2, 0x00200000 }, /* 2.0MB */\
    { kTivoConfigContigmemRegion8, 0x01600000 }, /*22.0MB */

#define BCM_UMA_HD_DTV_MEM \
    { kTivoConfigMemSize, 0x08000000 }, /* 128 MB */         \
    { kTivoConfigContigmemRegion1, 0x00C00000 }, /*12.0MB */\
    { kTivoConfigContigmemRegion2, 0x00200000 }, /* 2.0MB */\
    { kTivoConfigContigmemRegion8, 0x01600000 }, /*22.0MB */

#define BCM_UMA_HD_DTV \
    BCM_UMA_HD_DTV_MEM   				    \
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) }, \
    { kTivoConfigAnalogTunerCnt, 0 },                       \
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportDTV30 },  \
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportDTV },    \
    { kTivoConfigDigitalVideoOutputCnt, 1 },


#define BCM_UMA_DTV \
    BCM_UMA_DTV_MEM					    \
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) }, \
    { kTivoConfigDigitalTunerCnt, 2 },                      \
    { kTivoConfigAnalogTunerCnt, 0 },                       \
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportDTV30 },  \
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportDTV },    \
    { kTivoConfigDigitalVideoOutputCnt, 0 },


static TivoConfig gBrcmUma1aConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma1a },
    BCM_UMA_BASE
    BCM_UMA_SA
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gBrcmUma2ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma2 },
    { kTivoConfigI2cType, kTivoConfigI2cTypeDualMSBus },
    { kTivoConfigTempAdj, 0x01000000 }, /* unity scalar */
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |
                         kTivoConfigIdeCapAta66 |
                         kTivoConfigIdeCapSG |
                         kTivoConfigIdeCapScramble },
    BCM_UMA_SA
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gBrcmUma1bConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma1b },
    BCM_UMA_BASE
    BCM_UMA_SA_NO_FANCTL
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(73,62,60,50) },
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gBrcmUma1cConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma1c },
    BCM_UMA_BASE
    BCM_UMA_SA_NO_FANCTL
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(75,62,60,50) },
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gBrcmUma1dConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma1d },
    BCM_UMA_BASE
    BCM_UMA_SA_NO_FANCTL
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(65,62,60,50) },
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gBrcmUma1eConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma1e },
    BCM_UMA_BASE
    BCM_UMA_SA_NO_FANCTL
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(67,62,60,50) },
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gBrcmUma1fConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma1f },
    BCM_UMA_BASE
    BCM_UMA_SA_NO_FANCTL
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(69,62,60,50) },
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gBrcmUma3ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma3 },
    BCM_UMA_BASE
    BCM_UMA_DTV
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gBrcmUma4ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma4 },
    BCM_UMA_BASE
    BCM_UMA_DTV
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan4x4 },
#if 0 // XXX: temporary until all boards reworked...
    { kTivoConfigReset, kTivoConfigResetTivoAsicGpio9 },
#else
    { kTivoConfigReset, kTivoConfigResetUnknown },
#endif
    { 0,0 }
};
static TivoConfig gBrcmUma5ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma5 },
    BCM_UMA_BASE
    BCM_UMA_SA
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan4x4 },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gBrcmUma6ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma6 },
    BCM_UMA_BASE
    BCM_UMA_DTV
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan4x4 },
    { kTivoConfigReset, kTivoConfigResetUnknown },
    { 0,0 }
};
static TivoConfig gBrcmUma7PXConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma7PX },
    { kTivoConfigI2cType, kTivoConfigI2cTypeSingleMSBus },
    { kTivoConfigTempAdj, 0x01000000 }, /* unity scalar */
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |
                         kTivoConfigIdeCapSG |
                         kTivoConfigIdeCapScramble },
    { kTivoConfigContigmemRegion1, 0x00100000 }, /* 1.0MB */
    { kTivoConfigContigmemRegion2, 0x00080000 }, /* 512KB */
    { kTivoConfigContigmemRegion8, 0x00680000 }, /* 6.5MB */
    { kTivoConfigDigitalTunerCnt, 0 },
    { kTivoConfigAnalogTunerCnt, 1 },
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) },
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { 0,0 }
};
static TivoConfig gBrcmUma7ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBcmUma7 },
    { kTivoConfigI2cType, kTivoConfigI2cTypeSingleMSBus },
    { kTivoConfigTempAdj, 0x01000000 }, /* unity scalar */
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |
                         kTivoConfigIdeCapSG |
                         kTivoConfigIdeCapScramble },
    { kTivoConfigContigmemRegion1, 0x00100000 }, /* 1.0MB */
    { kTivoConfigContigmemRegion2, 0x00080000 }, /* 512KB */
    { kTivoConfigContigmemRegion8, 0x005bd000 }, /* 5.7MB */
    { kTivoConfigDigitalTunerCnt, 0 },
    { kTivoConfigAnalogTunerCnt, 1 },
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) },
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { 0,0 }
};

static TivoConfig gBrcmBrycePYConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBrycePY },
    { kTivoConfigI2cType, kTivoConfigI2cTypeSingleMSBus },
    { kTivoConfigTempAdj, 0x01000000 }, /* unity scalar */
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |
                         kTivoConfigIdeCapSG |
                         kTivoConfigIdeCapScramble },
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB XXX: s/b 64MB */
    /* Bryce PY is basically dead -- long live Bryce P0! */
    { kTivoConfigContigmemRegion1, 0x00400000 }, /* 4.0MB */
    { kTivoConfigContigmemRegion2, 0x00100000 }, /* 1.0MB */
    { kTivoConfigContigmemRegion8, 0x00a00000 }, /*10.0MB */
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) },
    { kTivoConfigDigitalTunerCnt, 2 },
    { kTivoConfigAnalogTunerCnt, 0 },
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportDTV30 },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportDTV },
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan4x4 },
    { 0,0 }
};
static TivoConfig gBrcmBryceConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDBryce },
    { kTivoConfigI2cType, kTivoConfigI2cTypeSingleMSBus },
    { kTivoConfigTempAdj, 0x01000000 }, /* unity scalar */
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |
                         kTivoConfigIdeCapSG |
                         kTivoConfigIdeCapScramble },
    { kTivoConfigMemSize, 0x04000000 }, /* s/b 64MB */
    { kTivoConfigContigmemRegion1, 0x00800000 }, /* 8.0MB */
    { kTivoConfigContigmemRegion2, 0x00100000 }, /* 1.0MB */
    { kTivoConfigContigmemRegion8, 0x00a00000 }, /*10.0MB */
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) },
    { kTivoConfigDigitalTunerCnt, 2 },
    { kTivoConfigAnalogTunerCnt, 0 },
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportDTV30 },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportDTV },
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan4x4 },
    { 0,0 }
};
static TivoConfig gPhoenixPX1ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDPhoenixPX1 },
    BCM_UMA_ALT_BASE
    BCM_UMA_HD_DTV_PX_MEM 
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) }, \
    { kTivoConfigAnalogTunerCnt, 0 },                       \
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportDTV30 },  \
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportDTV },
    { kTivoConfigDigitalVideoOutputCnt, 0 },              
    { kTivoConfigDigitalTunerCnt, 1 },                      \
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigReset, kTivoConfigResetUnknown },
    { 0,0 }
};
static TivoConfig gPhoenixPX2ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDPhoenixPX2 },
    BCM_UMA_ALT_BASE
    BCM_UMA_HD_DTV_PX_MEM
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) }, \
    { kTivoConfigAnalogTunerCnt, 0 },                       \
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportDTV30 },  \
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportDTV },
    { kTivoConfigDigitalVideoOutputCnt, 0 },
    { kTivoConfigDigitalTunerCnt, 2 },                      \
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigReset, kTivoConfigResetUnknown },
    { 0,0 }
};
static TivoConfig gPhoenixConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDPhoenix },
    BCM_UMA_BASE
    BCM_UMA_HD_DTV
    { kTivoConfigDigitalTunerCnt, 2 },                      \
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigReset, kTivoConfigResetUnknown },
    { 0,0 }
};
static TivoConfig gDefiantPX1ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDDefiantPX1 },
    BCM_UMA_BASE
    BCM_UMA_HD_SA
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gDefiantPX2ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDDefiantPX2 },
    BCM_UMA_BASE
    BCM_UMA_HD_SA
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigReset, kTivoConfigResetUnknown },
    { 0,0 }
};

static TivoConfig gFalconUS1ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDFalconUS1 },
    BCM_UMA_BASE
    BCM_UMA_SA
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};

/* Next tables are for the Takara platforms.
 *
 */

#define TAKARA_FAN                                              \
    { kTivoConfigFanCtl,  TEMPERATURE_PARAMS(71,62,60,50) },    \
    { kTivoConfigFanCtl2, TEMPERATURE_PARAMS(71,62,60,50) },    \
    { kTivoConfigFanLowSpeed, 5 },

#define BCM_UMA_TAKARA_BASE \
    BCM_UMA_BASE \
    { kTivoConfigContigmemRegion1, 0x00300000 }, /* 3.0MB */\
    { kTivoConfigContigmemRegion2, 0x00200000 }, /* 2.0MB */\
    { kTivoConfigContigmemRegion8, 0x00b00000 }, /* 11.0MB */\
    { kTivoConfigDigitalTunerCnt, 0 },                      \
    { kTivoConfigAnalogTunerCnt, 1 },                       \
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },   \
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },   \
    { kTivoConfigDigitalVideoOutputCnt, 0 },                \
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },    \
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },         \
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan4x4 },      \
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },     \
    TAKARA_FAN

/* Not using the TAKARA_BASE for PX's because they only have 32MB, so */
/* will be config'd differently (though likely won't run anyway .... */
static TivoConfig gTakaraPXConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDTakaraPX },
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */         
    BCM_UMA_BASE
    TAKARA_FAN
    { kTivoConfigContigmemRegion1, 0x00100000 }, /* 1.0MB */
    { kTivoConfigContigmemRegion2, 0x00080000 }, /* 512KB */
    { kTivoConfigContigmemRegion8, 0x00580000 }, /* 5.5MB */
    { kTivoConfigDigitalTunerCnt, 0 },                      
    { kTivoConfigAnalogTunerCnt, 1 },                       
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },   
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },   
    { kTivoConfigDigitalVideoOutputCnt, 0 },
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },  
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan4x4 },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gTakaraP0ConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDTakaraP0 },
    { kTivoConfigMemSize, 0x04000000 }, /* 64 MB */         
    BCM_UMA_TAKARA_BASE
    { 0,0 }
};
static TivoConfig gTakaraP1ConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDTakaraP1 },
    { kTivoConfigMemSize, 0x04000000 }, /* 64 MB */         
    BCM_UMA_TAKARA_BASE
    { 0,0 }
};
static TivoConfig gTakaraP2ConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDTakaraP2 },
    { kTivoConfigMemSize, 0x04000000 }, /* 64 MB */         
    BCM_UMA_TAKARA_BASE
    { 0,0 }
};
static TivoConfig gTakaraP2aConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDTakaraP2a },
    { kTivoConfigMemSize, 0x04000000 }, /* 64 MB */         
    BCM_UMA_TAKARA_BASE
    { 0,0 }
};
static TivoConfig gTakaraConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDTakara },
    { kTivoConfigMemSize, 0x04000000 }, /* 64 MB */         
    BCM_UMA_TAKARA_BASE
    { 0,0 }
};


/* Next tables are for the Toshiba platforms.
 *
 */
static TivoConfig gToshibaPXConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDToshibaPX },
    BCM_UMA_BASE
    BCM_UMA_SA
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan4x4 },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};
static TivoConfig gToshibaP0ConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDToshibaP0 },
    BCM_UMA_BASE
    BCM_UMA_SA
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan5x3 },
    { kTivoConfigReset, kTivoConfigResetBcm7020Gpio0 },
    { 0,0 }
};

/* Elmo, similar to Ibis and Takara */
static TivoConfig gElmoP0ConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDElmoP0 },
    { kTivoConfigI2cType, kTivoConfigI2cTypeSingleMSBus },
    { kTivoConfigTempAdj, 0x01000000 }, /* unity scalar */
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |
                         kTivoConfigIdeCapSG |
                         kTivoConfigIdeCapScramble },
    { kTivoConfigContigmemRegion1, 0x001E0000 }, /* ~1.9MB */
    { kTivoConfigContigmemRegion2, 0x00080000 }, /* 512KB */
    { kTivoConfigContigmemRegion8, 0x005bd000 }, /* 5.7MB */
    { kTivoConfigDigitalTunerCnt, 0 },
    { kTivoConfigAnalogTunerCnt, 1 },
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) },
    { kTivoConfigFanCtl2, TEMPERATURE_PARAMS(71,62,60,50) },
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { 0,0 }
};

/* Oscar, similar to Elmo P0 */
static TivoConfig gOscarConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDOscar },
    { kTivoConfigI2cType, kTivoConfigI2cTypeSingleMSBus },
    { kTivoConfigTempAdj, 0x01000000 }, /* unity scalar */
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma |
                         kTivoConfigIdeCapSG |
                         kTivoConfigIdeCapScramble },
    { kTivoConfigContigmemRegion1, 0x001E0000 }, /* ~1.9MB */
    { kTivoConfigContigmemRegion2, 0x00080000 }, /* 512KB */
    { kTivoConfigContigmemRegion8, 0x005bd000 }, /* 5.7MB */
    { kTivoConfigDigitalTunerCnt, 0 },
    { kTivoConfigAnalogTunerCnt, 1 },
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },
    { kTivoConfigFanCtl, TEMPERATURE_PARAMS(71,62,60,50) },
    { kTivoConfigFanCtl2, TEMPERATURE_PARAMS(71,62,60,50) },
    { kTivoConfigMemSize, 0x02000000 }, /* 32 MB */
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { 0,0 }
};

/* Eiger PX is a Broadcom 97038 Hardware Reference Platform */
static TivoConfig gEigerPXConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDEigerPX },
    { kTivoConfigMemSize, 0x08000000 }, // 128 MB
    { kTivoConfigI2cType, kTivoConfigI2cTypeQuadMSBus },
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma | 
                         kTivoConfigIdeCapSATA | 
			 kTivoConfigIdeCapSG },
    { kTivoConfigTempAdj, 0x01000000 },
    { kTivoConfigContigmemRegion1, 0x00C00000 }, /*12.0MB */
    { kTivoConfigContigmemRegion2, 0x00100000 }, /* 1.0MB */
    { kTivoConfigContigmemRegion8, 0x04000000 }, /*64.0MB */
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },
    { kTivoConfigFanCheck, kTivoConfigFanCheckNone },
    { kTivoConfigDigitalTunerCnt, 2 },
    { kTivoConfigAnalogTunerCnt, 1 },
    { kTivoConfigDigitalVideoOutputCnt, 1 },
    { kTivoConfigIrMicro, kTivoConfigIrMicroPresent },
    { kTivoConfigReset, kTivoConfigResetEigerPx },
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan5and6 },
    { 0,0 }
};

/* Eiger P0 is TiVo 7038 with dual analog + POD inputs */
static TivoConfig gEigerP0ConfigTable[] __initdata = {
    { kTivoConfigBoardID, kTivoConfigBoardIDEiger },
    { kTivoConfigMemSize, 0x08000000 }, // 128 MB
    { kTivoConfigI2cType, kTivoConfigI2cTypeQuadMSBus },
    { kTivoConfigIdeCap, kTivoConfigIdeCapUdma | 
                         kTivoConfigIdeCapSATA | 
			 kTivoConfigIdeCapSG },
    { kTivoConfigTempAdj, 0x01000000 },
    { kTivoConfigContigmemRegion1, 0x00C00000 }, /*12.0MB */
    { kTivoConfigContigmemRegion2, 0x00200000 }, /* 2.0MB */
    { kTivoConfigContigmemRegion8, 0x04000000 }, /*64.0MB */
    { kTivoConfigAPGSupport, kTivoConfigAPGSupportNone },
    { kTivoConfigCAMSupport, kTivoConfigCAMSupportNone },
    { kTivoConfigFanCheck, kTivoConfigFanCheckPresent },
    { kTivoConfigDigitalTunerCnt, 2 },
    { kTivoConfigAnalogTunerCnt, 2 },
    { kTivoConfigDigitalVideoOutputCnt, 1 },
    { kTivoConfigIrMicro, kTivoConfigIrMicroNone },
    { kTivoConfigReset, kTivoConfigResetEigerPx },
    { kTivoConfigFPKeyScan, kTivoConfigFPKeyScan5and6 }, /* probably wrong */
    { 0,0 }
};


/* The Unknown Platform.
 */
static TivoConfig gUnknown128MConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDUnknown128M },
    BCM_UMA_BASE
    { kTivoConfigMemSize, 0x08000000 }, // 128 MB
    { kTivoConfigContigmemRegion1, 0x00C00000 }, /*12.0MB */\
    { kTivoConfigContigmemRegion2, 0x00100000 }, /* 1.0MB */\
    { kTivoConfigContigmemRegion8, 0x01600000 }, /*22.0MB */\
    { 0,0 }
};
static TivoConfig gUnknown64MConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDUnknown64M },
    BCM_UMA_BASE
    { kTivoConfigMemSize, 0x04000000 }, // 64 MB
    { kTivoConfigContigmemRegion1, 0x00800000 }, /* 8.0MB */\
    { kTivoConfigContigmemRegion2, 0x00100000 }, /* 1.0MB */\
    { kTivoConfigContigmemRegion8, 0x00a00000 }, /*10.0MB */\
    { 0,0 }
};
static TivoConfig gUnknown32MConfigTable[] __initdata = { 
    { kTivoConfigBoardID, kTivoConfigBoardIDUnknown32M },
    BCM_UMA_BASE
    { kTivoConfigMemSize, 0x02000000 }, // 32 MB
    { kTivoConfigContigmemRegion1, 0x00100000 }, /* 1.0MB */\
    { kTivoConfigContigmemRegion2, 0x00080000 }, /* 512KB */\
    { kTivoConfigContigmemRegion8, 0x00580000 }, /* 5.5MB */
    { 0,0 }
};

static TivoConfig *gGlobalConfigTable[] __initdata = { 
    gClassicConfigTable,
    gComboConfigTable,
    gBrcmUma1aConfigTable,
    gBrcmUma2ConfigTable,
    gBrcmUma1bConfigTable,
    gBrcmUma1cConfigTable,
    gBrcmUma1dConfigTable,
    gBrcmUma1eConfigTable,
    gBrcmUma1fConfigTable,
    gBrcmUma3ConfigTable,
    gBrcmUma4ConfigTable,
    gBrcmUma5ConfigTable,
    gBrcmUma6ConfigTable,
    gBrcmUma7PXConfigTable,
    gBrcmUma7ConfigTable,
    gElmoP0ConfigTable,
    gOscarConfigTable,
    gBrcmBrycePYConfigTable,
    gBrcmBryceConfigTable,
    gPhoenixPX1ConfigTable,
    gPhoenixPX2ConfigTable,
    gPhoenixConfigTable,
    gDefiantPX1ConfigTable,
    gDefiantPX2ConfigTable,
    gFalconUS1ConfigTable,
    gTakaraPXConfigTable,
    gTakaraP0ConfigTable,
    gTakaraP1ConfigTable,
    gTakaraP2ConfigTable,
    gTakaraP2aConfigTable,
    gTakaraConfigTable,
    gToshibaPXConfigTable,
    gToshibaP0ConfigTable,
    gEigerPXConfigTable,
    gEigerP0ConfigTable,
    gUnknown128MConfigTable,
    gUnknown64MConfigTable,
    gUnknown32MConfigTable,
    0
};

#else

#error "Nobody should ever #include <linux/tivoconfig-init.h> more than once!"

#endif /* _LINUX_TIVOCONFIG_INIT_H */
